/* Simple implementation to expand the pixel data of Poly-Raster bitmaps.
 * The program that calls the unrle() function must declare a variable of
 * the type "struct rlestat" and initialize both fields to zero.
 *
 * Function unrle() returns a single byte from the stream after each call.
 * This byte may either come from the file, or from the cache in struct rlestat.
 *
 * Function unrle() depends on a macro GETBYTE() to read the next byte, either
 * from a file or from a memory region.
 */

struct rlestat {
  unsigned char prev;
  unsigned char count;
};

unsigned char unrle(void *input, struct rlestat *stat)
{
  unsigned char value;
  if (stat->count > 0) {
    stat->count -= 1;
    value = stat->prev;
  } else {
    value = (unsigned char)GETBYTE(input);
    if (value == stat->prev)
      stat->count = (unsigned char)GETBYTE(input);
    else
      stat->prev = value;
  } /* if */
  return value;
}
